#!/usr/bin/env ruby

# $B$^$@$&$4$+$J$$$h(B

# $Id: mgp2lua.rb,v 1.4 2004/05/01 02:42:37 zinnia Exp $

DEFAULT_PATH="/usr/X11R6/share/examples/mgp";

class Context
  attr_reader :contflag, :halign, :linecount, :default_no, :tab_no, :defaults,
    :tab, :size, :nodefault

  attr_writer :defaults, :default_no,  :tab, :tab_no, :linecount, :halign, :size, :nodefault, :default_no, :tab_no

  def initialize()
    @contflag = 0
    @nodefault = 0
    @halign = "left"
    @linecount = 0

    @default_no = -1
    @tab_no = -1

    @size = 0
    @defaults = Array.new()
    @tab = Array.new()
    
  end
end

def register(expression, context)
  if context.default_no != -1 then

    if context.defaults[context.default_no] == nil then context.defaults[context.default_no] = '' end
    context.defaults[context.default_no] = context.defaults[context.default_no] + expression + "\n"

  elsif context.tab_no != -1 then
    if context.tab[context.tab_no] == nil then context.tab[context.tab_no] = '' end
    context.tab[context.tab_no] = context.tab[context.tab_no] + expression + "\n"
  else
    print expression + "\n"
  end
end

def parse_element(elem, context)

  if elem =~ /^[ \t]*([^ \t]+)[ \t]*/ then
    if $1 == "page" then
      if context.linecount > 0 then
	register("sps_endpage()\n", context)
	context.halign = "left"
      end
      register("sps_beginpage()\nsps_clear()\n", context);
      context.linecount = 0
      context.nodefault = 0
    elsif $1 == "include" then
      parse("#{DEFAULT_PATH}/#{$'.split(/\"/).join}", context)
    elsif $1 == "center" then
      context.halign = "center"
    elsif $1 == "left" then
      context.hlign = "left"
    elsif $1 == "right" then
      context.halign = "right"
    elsif $1 == "size" then
    elsif $1 == "vgap" then
      register("sps_moveto(0, #{$'.split(/\"/).join})\n", context)
    elsif $1 == "prefix" then
      register("sps_puttext(#{$'}, #{context.size})", context)
    elsif $1 == "icon" then
    elsif $1 == "nodefault" then
      context.nodefault = 1
    elsif $1 == "fore" then
      register("sps_forecolor(#{$'})", context);
    elsif $1 == "back" then
      register("sps_backcolor(#{$'})", context);
    else
      STDERR.print "#{$1}: unknown directive.\n"
    end
  end
end

def parse_controlline(line, context)
  if line =~ /^%/ then return end
  if line =~ /^[ \t]*([^ \t]+)/ then
    if $1 == "default" then
      if line =~ /^[ \t]*default[ \t]+([0-9]+)[ \t]+/ then
	context.default_no = Integer($1)
	parse_controlline($', context)
	context.default_no = -1
      end
    elsif $1 == "tab" then
      if line =~ /^[ \t]*tab[ \t]+([0-9]+)[ \t]+/ then
	context.tab_no = Integer($1)
	parse_controlline($', context)
	context.tab_no = -1
      end
    else
      list = line.split(/,/)
      list.each { |elem|
	parse_element(elem, context)
      }
    end
  end
end

def parse(filename, context)
  open(filename) {|p|
    p.readlines.each {|line|
      line.chomp!
      if line =~ /^%/ then
	parse_controlline($', context)
      else
	if context.nodefault == 0 && context.defaults[context.linecount] != nil then
	  print context.defaults[context.linecount] + "\n"
	end
	if line =~ /^([\t]+)/ then
	  l = $1.length;
	  if context.nodefault == 0 && context.tab[l] != nil then
	    print context.tab[l] + "\n"
	  end
	  line = $'
	end
	
	print "sps_beginline(\"" + context.halign + "\")\n"
	print "sps_puttext(\"#{line}\")\n"
	if context.contflag == 0 then
	  print "sps_endline()\n"
	
	  context.linecount = context.linecount + 1
	end
      end
    }
  }
end

parse(ARGV.shift, Context.new())
